
__all__ = ['cmd', 'env', 'LaTeX', 'tolatex', 'Command', 'Environment',
        'usepackage']

class Command(object):
    def __init__(self, name):
        self.name = name

    def begin(self, optional=None, mandatory=None):
        result = LaTeX()
        result += '\\'+self.name
        if optional is not None:
            result += '[%s]' % self.build_args(optional)
        if mandatory is not None:
            result += '{%s}' % self.build_args(mandatory)
        result += '{'
        return result

    def end(self):
        return LaTeX('}')

    def __call__(self, content=None, optional=None, mandatory=None):
        return self.begin(optional, mandatory) + tolatex(content) + self.end() + '\n'

    def __latex__(self):
        return self()

    def build_args(self, args):
        return ','.join(tolatex(arg) for arg in args)


class Environment(Command):
    def begin(self, optional=None, mandatory=None):
        result = LaTeX()
        result += '\\begin{%s}' % self.name
        if optional is not None:
            result += '[%s]' % self.build_args(optional)
        if mandatory is not None:
            result += '{%s}' % self.build_args(mandatory)
        result += '\n'
        return result

    def end(self):
        result = '\\end{%s}' % self.name
        result += '\n'
        return result

    def __call__(self, content=None, optional=None, mandatory=None):
        return self.begin(optional, mandatory) + tolatex(content) + self.end()


class LaTeX(unicode):
    def __latex__(self):
        return self


class Factory(object):
    def __init__(self, cls):
        self.cls = cls

    def __getattr__(self, name):
        return self.cls(name)

cmd = Factory(Command)
env = Factory(Environment)

def usepackage(package, options=None):
    return cmd.usepackage(package, 
            optional=([options] if options is not None else None))

def tolatex(s):
    if not s:
        return ''
    elif hasattr(s, '__latex__'):
        return s.__latex__()
    elif isinstance(s, basestring):
        return encode(s)
    elif hasattr(s, 'astext'):
        return tolatex(s.astext())
    else:
        return ''.join(tolatex(item) for item in s)

def encode(s):
    s = s.replace('<', r'{\textless}')
    s = s.replace('>', r'{\textgreater}')
    s = s.replace('|', r'{\textbar}')
    return s
